#include "GBase.h"
#include "GArgs.h"
#include "GStr.h"
#include "GList.hh"
#include "GAlnExtend.h"

#define USAGE "Usage:\n\
   seqalign {-i <2line_file> | -a <seq1> -b <seq2>} \\ \n\
      [X=<xdrop>] [M=<match_reward>] [N=<mismatch_penalty]\
   "

bool debug=true;
int Xdrop=8;
int match_reward=2;
int mismatch_penalty=3;

#define FRCLOSE(fh) if (fh!=NULL && fh!=stdin) fclose(fh)
#define FWCLOSE(fh) if (fh!=NULL && fh!=stdout) fclose(fh)

char* seq_cleanup(const char* s, int& s_len) {
 char* r=NULL;
 int slen=strlen(s);
 GMALLOC(r, slen+1);
 int newlen=0;
 for (int i=0;i<slen;i++) {
   if (s[i]<'A' || s[i]>'z' || (s[i]>'Z' && s[i]<'a')) continue;
   r[newlen]=s[i];
   newlen++;
   }
 r[newlen]=0;
 s_len=newlen;
 return r;
}

bool loadSeqs(GLineReader& lr, char*& s1, char*& s2,
               int& s1_len, int& s2_len) {
 char* l=NULL;
 s1=NULL;
 s2=NULL;
 while ((l=lr.nextLine())!=NULL) {
  if (lr.length()<=3 || l[0]=='#') continue;
  if (s1==NULL) {
    s1=seq_cleanup(l, s1_len);
    }
   else if (s2==NULL) {
             s2=seq_cleanup(l, s2_len);
             return true;
             }
           else break;
  }
 if (s1!=NULL) GFREE(s1);
 return false;
}

void findTrimming(char* aseq, int alen, char* rseq, int rlen, CGreedyAlignData* gxmem_l=NULL, CGreedyAlignData* gxmem_r=NULL) {


 GXAlnInfo* r_bestaln=match_RightEnd(aseq, alen, rseq, rlen, gxmem_r);

 #ifdef GDEBUG
  if (r_bestaln)
  GMessage("******** Best 3' Alignment: %d-%d vs %d-%d, score=%d, pid=%.2f\n",
        r_bestaln->ql, r_bestaln->qr, r_bestaln->sl, r_bestaln->sr, r_bestaln->score, r_bestaln->pid);
  else GMessage("******** No suitable 3' alignment found.\n");
 #endif
  if (r_bestaln)
    fprintf(stdout, "3prime %d-%d:%d-%d|score=%d|pid=%.2f\n",
      r_bestaln->ql, r_bestaln->qr, r_bestaln->sl, r_bestaln->sr, r_bestaln->score, r_bestaln->pid);

  delete r_bestaln;

  //testing: reverse both sequences and redo the search at 5' end
  reverseChars(aseq);
  reverseChars(rseq);

  GXAlnInfo* l_bestaln=match_LeftEnd(aseq, alen, rseq, rlen, gxmem_l);

 #ifdef GDEBUG
  if (l_bestaln)
    GMessage("******** Best 5' Alignment: %d-%d vs %d-%d, score=%d, pid=%.2f\n",
        l_bestaln->ql, l_bestaln->qr, l_bestaln->sl, l_bestaln->sr, l_bestaln->score, l_bestaln->pid);
    else GMessage("******** No suitable 5' alignment found.\n");
 #endif
  if (l_bestaln)
    fprintf(stdout, "5prime %d-%d:%d-%d|score=%d|pid=%.2f\n",
      l_bestaln->ql, l_bestaln->qr, l_bestaln->sl, l_bestaln->sr, l_bestaln->score, l_bestaln->pid);

  delete l_bestaln;

  GFREE(aseq);
  GFREE(rseq);
}

//================= MAIN =====================

int main(int argc, char * const argv[]) {
 GArgs args(argc, argv, "X=M=N=a:b:i:");
 int e;
 if ((e=args.isError())>0)
    GError("%s\nInvalid argument: %s\n", USAGE, argv[e]);
 if (args.getOpt('h')!=NULL) GError("%s\n", USAGE);
 char* seqa=NULL;
 char* seqb=NULL;
 GStr s=args.getOpt('X');
 if (!s.is_empty())
    Xdrop=s.asInt();
 s=args.getOpt('M');
 if (!s.is_empty())
    match_reward=s.asInt();
 s=args.getOpt('N');
 if (!s.is_empty())
    mismatch_penalty=s.asInt();

 s=args.getOpt('i');
 int seqa_len=0, seqb_len=0;
 CGreedyAlignData* gxmem_l=new CGreedyAlignData(match_reward, mismatch_penalty, Xdrop-2);
 CGreedyAlignData* gxmem_r=new CGreedyAlignData(match_reward, mismatch_penalty, Xdrop);

 if (!s.is_empty()) {
      GLineReader lr(s.chars());
      while (loadSeqs(lr, seqa, seqb, seqa_len, seqb_len)) {
         findTrimming(seqa, seqa_len, seqb, seqb_len, gxmem_l, gxmem_r);
         }
      }
   else {
     s=args.getOpt('a');
     if (!s.is_empty()) seqa=seq_cleanup(s.chars(), seqa_len);
     s=args.getOpt('b');
     if (!s.is_empty()) seqb=seq_cleanup(s.chars(), seqb_len);
     if (seqa==NULL || seqa_len<2 || seqb==NULL || seqb_len<2)
        GError("Error: two sequences with length>1 should be given!\n");
     findTrimming(seqa, seqa_len, seqb, seqb_len, gxmem_l, gxmem_r);
     }
 delete gxmem_l;
 delete gxmem_r;

}


